unit fMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TBreakEvent = procedure(Sender: TObject; var IsBreak: Boolean) of object;

  TMyThread = class(TThread)
  private
    FOnStart: TNotifyEvent;
    FOnDone: TNotifyEvent;
    FOnWorking: TNotifyEvent;
    FOnCheckBreak: TBreakEvent;
    FCountDown: Integer;
    FIndex: Integer;
    FOwner: TForm;
    FOnBreak: TNotifyEvent;
  protected
    FX: Integer;
    FY: Integer;
    FColor: TColor;
    procedure Execute; override;
    procedure Draw;
  public
    constructor Create(AOwner: TForm; Index, CountDown: Integer);
    property Index: Integer read FIndex;
    property Owner: TForm read FOwner;
    property CountDown: Integer read FCountDown;
    property OnStart: TNotifyEvent read FOnStart write FOnStart;
    property OnWorking: TNotifyEvent read FOnWorking write FOnWorking;
    property OnDone: TNotifyEvent read FOnDone write FOnDone;
    property OnCheckBreak: TBreakEvent read FOnCheckBreak write FOnCheckBreak;
    property OnBreak: TNotifyEvent read FOnBreak write FOnBreak;
  end;

  TForm1 = class(TForm)
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  private
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TMyThread }

constructor TMyThread.Create(AOwner: TForm; Index, CountDown: Integer);
begin
  inherited Create(True);
  FreeOnTerminate := True;
  FOwner := AOwner;
  FIndex := Index;
  FCountDown := CountDown;
  FX := (FIndex mod 5) * 150;
  FY := (FIndex div 5) * 20;
  FColor := RGB(Random($100),Random($100),Random($100));
  Resume;
end;

procedure TMyThread.Draw;
var
  C: TCanvas;
  S: string;
  R: TRect;
begin
  if FOwner <> nil then
  begin
    S := Format('%d@(%d,%d)=%d',[FIndex,FX,FY,FCountDown]);
    C := FOwner.Canvas;
    C.Lock;
    try
      C.Brush.Color := FOwner.Color;
      C.Brush.Style := bsSolid;
      C.Font.Color := FColor;
      C.Font.Height := 20 - 2;
      R := Rect(0,0,150,C.TextHeight('0'));
      OffsetRect(R,FX+100,FY);
      C.TextRect(R,FX+100,FY,S);
    finally
      C.Unlock;
    end;
  end;
end;

procedure TMyThread.Execute;
var
  IsBreak: Boolean;
begin
  IsBreak := False;

  if Assigned(FOnStart) then FOnStart(Self);
  while not Terminated do
  begin
    Synchronize(Draw);

    if Assigned(FOnCheckBreak) then FOnCheckBreak(Self, IsBreak);

    if IsBreak then Break;

    if Assigned(FOnWorking) then FOnWorking(Self);

    Dec(FCountDown);

    if FCountDown <= 0 then Break;

    Application.ProcessMessages;
    Sleep(10);
    
  end;

  Synchronize(Draw);

  case IsBreak of
    False: if Assigned(FOnDone) then FOnDone(Self);
    else   if Assigned(FOnBreak) then FOnBreak(Self);
  end;

  Terminate;

end;

{ TForm1 }

procedure TForm1.Button1Click(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to 149 do
    TMyThread.Create(Self,I,Random(200)+200);
end;

end.
